package com.huigou.topsun.ep.change.application.impl;

import com.huigou.data.domain.model.CommonDomainConstants;
import com.huigou.data.query.model.QueryDescriptor;
import com.huigou.data.query.model.QueryModel;
import com.huigou.topsun.ep.change.application.EpChangeFormApplication;
import com.huigou.topsun.ep.change.application.EpChangeFormImplItemApplication;
import com.huigou.topsun.ep.change.application.EpChangeFormStockItemApplication;
import com.huigou.topsun.ep.change.domain.EpChangeForm;
import com.huigou.topsun.ep.change.domain.EpChangeFormImplItem;
import com.huigou.topsun.ep.change.domain.EpChangeFormStockItem;
import com.huigou.topsun.ep.change.domain.query.EpChangeFormQueryRequest;
import com.huigou.topsun.ep.change.repository.EpChangeFormRepository;
import com.huigou.uasp.bmp.common.BizBillStatus;
import com.huigou.uasp.bpm.FlowBroker;
import com.huigou.util.ClassHelper;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.delegate.DelegateTask;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.List;
import java.util.Map;

/**
 * @Auther: xin.lu
 * @Date: 2024/06/19/10:06
 * @Description:
 */
@Service("epChangeFormApplication")
public class EpChangeFormApplicationImpl extends FlowBroker implements EpChangeFormApplication {
    @Autowired
    private EpChangeFormRepository epChangeFormRepository;
    @Autowired
    private EpChangeFormStockItemApplication changeFormStockItemApplication;
    @Autowired
    private EpChangeFormImplItemApplication changeFormImplItemApplication;
    @Override
    protected String saveBizAndApprovalData() {
        super.saveBizAndApprovalData();
        EpChangeForm epChangeForm = getBizEntity(EpChangeForm.class);
        if (epChangeForm.isNew()) {
            epChangeForm.setStatusId(BizBillStatus.APPLYING.getId());
        } else {
            epChangeForm = (EpChangeForm) commonDomainService.loadAndFillinProperties(epChangeForm);
        }
        epChangeForm = epChangeFormRepository.save(epChangeForm);
        List<EpChangeFormStockItem> changeFormStockItems = getBizEntities(EpChangeFormStockItem.class, "epChangeFormStockItem");
        changeFormStockItemApplication.saveEpChangeFormStockItems(epChangeForm.getId(),changeFormStockItems);

        List<EpChangeFormImplItem> changeFormImplItems = getBizEntities(EpChangeFormImplItem.class, "EpChangeFormImplItem");
        changeFormImplItemApplication.saveEpChangeFormImplItems(epChangeForm.getId(),changeFormImplItems);
        return epChangeForm.getId();
    }

    @Override
    protected Map<String, Object> getProcessBizParams(String bizId) {
        // 返回业务数据给流程实例，
        return ClassHelper.toMap(epChangeFormRepository.getOne(bizId));
    }

    /**
     * 流程撤销事件
     **/
    @Override
    protected void onRecallProcessInstance(DelegateExecution delegateExecution) {
        String bizId = delegateExecution.getProcessBusinessKey();
        // 修改单据状态为申请
        updateStatus(bizId, BizBillStatus.APPLYING);
    }

    /**
     * 任务完成前执行
     */
    @Override
    protected void onBeforeComplete(DelegateTask delegateTask) {
        super.onBeforeComplete(delegateTask);
        String bizId = delegateTask.getExecution().getProcessBusinessKey();
        if (this.isApplyProcUnit(delegateTask) && this.getApprovalParameter().isAdvanceProcessAction()) {
            updateStatus(bizId, BizBillStatus.APPROVING);
        }
    }

    /**
     * 回收事件
     **/
    @Override
    protected void onWithdraw(DelegateTask delegateTask, String destActivityId) {
        super.onWithdraw(delegateTask, destActivityId);
        String bizId = delegateTask.getExecution().getProcessBusinessKey();
        if (destActivityId.equalsIgnoreCase("apply")) {
            updateStatus(bizId, BizBillStatus.APPLYING);
        }
    }

    @Transactional
    private void updateStatus(String bizId, BizBillStatus status) {
        Assert.hasText(bizId, CommonDomainConstants.ID_NOT_BLANK);
        this.commonDomainService.updateStatus(EpChangeForm.class, bizId, status.getId());
    }

    @Override
    protected void onEnd(DelegateExecution delegateExecution) {
        super.onEnd(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        BizBillStatus status = approvePassed() ? BizBillStatus.COMPLETED : BizBillStatus.ABORTED;
        EpChangeForm epChangeForm = epChangeFormRepository.findOne(bizId);
        epChangeForm.setStatusId(status.getId());
        epChangeFormRepository.save(epChangeForm);
    }

    @Override
    protected void onAbortProcessInstance(DelegateExecution delegateExecution) {
        super.onAbortProcessInstance(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        EpChangeForm epChangeForm = epChangeFormRepository.findOne(bizId);
        epChangeForm.setStatusId(BizBillStatus.ABORTED.getId());
        epChangeFormRepository.save(epChangeForm);
    }

    /**
     * @param delegateTask
     * @param destActivityId 回退到的目标节点id
     */
    @Override
    protected void onBack(DelegateTask delegateTask, String destActivityId) {
        //super.onBack(delegateTask, destActivityId);
        if ("Apply".equalsIgnoreCase(destActivityId)) {
            String bizId = delegateTask.getExecution().getProcessBusinessKey();
            EpChangeForm epChangeForm = epChangeFormRepository.findOne(bizId);
            epChangeForm.setStatusId(BizBillStatus.APPLYING.getId());
            epChangeFormRepository.save(epChangeForm);
        }
    }

    @Override
    public EpChangeForm findEpChangeFormById(String id) {
        return epChangeFormRepository.findOne(id);
    }

    @Override
    public EpChangeForm saveEpChangeForm(EpChangeForm epChangeForm) {
        return epChangeFormRepository.save(epChangeForm);
    }

    @Override
    public Map<String, Object> slicedEpChangeFormList(EpChangeFormQueryRequest queryRequest) {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "epChangeForm");
        QueryModel queryModel = this.sqlExecutorDao.getQueryModel(queryDescriptor, queryRequest);
        queryModel.putDictionary("status", BizBillStatus.getMap());
        Map<String, Object> map = this.sqlExecutorDao.executeSlicedQuery(queryModel);
        return map;
    }
}
